pub struct ChunkPrefetchOptions {
pub max_fetch_stage: FetchStage,
pub max_on_wire_bytes_per_batch: u64,
pub max_bytes_on_wire_at_once: u64,
}Expand description
How to calculate which chunks to prefetch.
Fields
max_fetch_stage: FetchStageOnly prefetch chunks up to (and including) this stage.
Useful for debugging and for users who want to limit how aggressively we prefetch data ahead of what is strictly needed.
max_on_wire_bytes_per_batch: u64Batch together requests until we reach this size.
max_bytes_on_wire_at_once: u64Maximum number of bytes in transit at once.