pub struct StoreBundle { /* private fields */ }Expand description
Stores many EntityDbs of recordings and blueprints.
The stores are kept and iterated in insertion order to allow the UI to display them by default in opening order.
Implementations
Sourceimpl StoreBundle
impl StoreBundle
Sourcepub fn from_rrd<R: Read>(reader: BufReader<R>) -> Result<Self, StoreLoadError>
pub fn from_rrd<R: Read>(reader: BufReader<R>) -> Result<Self, StoreLoadError>
Decode an rrd stream. It can theoretically contain multiple recordings, and blueprints.
Sourcepub fn entity_dbs(&self) -> impl Iterator<Item = &EntityDb>
pub fn entity_dbs(&self) -> impl Iterator<Item = &EntityDb>
All loaded EntityDb, both recordings and blueprints, in insertion order.
Sourcepub fn entity_dbs_mut(&mut self) -> impl Iterator<Item = &mut EntityDb>
pub fn entity_dbs_mut(&mut self) -> impl Iterator<Item = &mut EntityDb>
All loaded EntityDb, both recordings and blueprints, in insertion order.
pub fn remove(&mut self, id: &StoreId) -> Option<EntityDb>
pub fn contains(&self, id: &StoreId) -> bool
pub fn get(&self, id: &StoreId) -> Option<&EntityDb>
pub fn get_mut(&mut self, id: &StoreId) -> Option<&mut EntityDb>
Sourcepub fn entry(&mut self, id: &StoreId) -> &mut EntityDb
pub fn entry(&mut self, id: &StoreId) -> &mut EntityDb
Returns either a recording or blueprint EntityDb.
One is created if it doesn’t already exist.
Sourcepub fn blueprint_entry(&mut self, id: &StoreId) -> &mut EntityDb
pub fn blueprint_entry(&mut self, id: &StoreId) -> &mut EntityDb
Creates one if it doesn’t exist.
Like Self::entry but also sets StoreInfo to a default value.
pub fn insert(&mut self, entity_db: EntityDb)
Sourcepub fn recordings(&self) -> impl Iterator<Item = &EntityDb>
pub fn recordings(&self) -> impl Iterator<Item = &EntityDb>
In insertion order.
Sourcepub fn recordings_mut(&mut self) -> impl Iterator<Item = &mut EntityDb>
pub fn recordings_mut(&mut self) -> impl Iterator<Item = &mut EntityDb>
In insertion order.
pub fn retain(&mut self, f: impl FnMut(&EntityDb) -> bool)
Sourcepub fn drain_entity_dbs(&mut self) -> impl Iterator<Item = EntityDb> + '_
pub fn drain_entity_dbs(&mut self) -> impl Iterator<Item = EntityDb> + '_
In insertion order.